@@ -11,18 +11,17 @@ from PyQt5.QtCore import QEvent
 from PyQt5.QtWidgets import QInputDialog, QMenu, QFileDialog, QAction, QMessageBox, QTreeWidget, QProgressBar, \
     QTableWidget
 
-from packetvisualization.backend_components.entity_operator import EntityOperations
+from packetvisualization.backend_components.entity_operator import EntityOperator
 from packetvisualization.backend_components.load import Load
 from packetvisualization.models import dataset
-# from packetvisualization.models.context.entities import EntityOperations
+# from packetvisualization.backend.context.entities import EntityOperations
 from packetvisualization.models.dataset import Dataset
 from packetvisualization.models.pcap import Pcap
 from packetvisualization.models.project import Project
 from packetvisualization.models.workspace import Workspace
 from packetvisualization.backend_components import Wireshark
 from packetvisualization.ui_components.table_gui import table_gui
-from packetvisualization.models.dataset_ent import Dataset as datasetEnt
-from packetvisualization.models.pcap_ent import Pcap as pcapEnt
+
 
 
 class Workspace_UI(QtWidgets.QMainWindow):
@@ -95,7 +94,7 @@ class Workspace_UI(QtWidgets.QMainWindow):
             self.progress_bar = QProgressBar(self)
             self.progress_bar.setGeometry(15, 518, 221, 23)
 
-            self.ent_operator = EntityOperations()
+            self.ent_operator = EntityOperator()
 
             if existing_flag:
                 self.generate_existing_workspace()
