@@ -7,84 +7,72 @@ import webbrowser
 import zipfile
 
 import plotly.offline as po
+import plotly.express as px
 import plotly.graph_objs as go
 import pandas as pd
 from PyQt5.QtWebEngineWidgets import QWebEngineView
 from scapy.all import *
 from datetime import datetime
 
-#from PyQt5.QtWebEngineWidgets import QWebEngineView
-
 from PyQt5.QtCore import Qt, QRect, QObject, pyqtSignal, QThread
 from PyQt5.QtGui import QFont, QIcon, QKeySequence
 from PyQt5.QtWidgets import QMainWindow, QTreeWidget, QPushButton, QVBoxLayout, QProgressBar, QMenu, QWidget, QLabel, \
-    QAction, QMessageBox, QDockWidget, QTextEdit, QInputDialog, QTreeWidgetItem, QFileDialog, QApplication, QToolBar
+    QAction, QMessageBox, QDockWidget, QTextEdit, QInputDialog, QTreeWidgetItem, QFileDialog, QApplication, QToolBar, \
+    QTableWidgetItem
 
+from packetvisualization.backend_components.bandwidth_plot import create_plot
+from packetvisualization.backend_components.controller import Controller
+from packetvisualization.backend_components.entity_operator import EntityOperations
 from packetvisualization.backend_components.load import Load
-# from packetvisualization.models.context.entities import EntityOperations
+from packetvisualization.models.context.database_context import DbContext
 from packetvisualization.models.dataset import Dataset
 from packetvisualization.models.pcap import Pcap
 from packetvisualization.models.project import Project
 from packetvisualization.models.workspace import Workspace
 from packetvisualization.backend_components import Wireshark
 from packetvisualization.ui_components import filter_gui
+from packetvisualization.ui_components.plot_worker import PlotWorker
 from packetvisualization.ui_components.table_gui import table_gui
 
 
-class Worker(QObject):
-    finished = pyqtSignal()
-    progress = pyqtSignal(int)
-    data = pyqtSignal(list)
-
-    def __init__(self, range, r_val, pcap: str = ''):
-        super().__init__()
-        self.range = range
-        self.r_val = r_val
-        self.pcap = pcap
-
-    def run(self):
-        if self.pcap != '':
-            self.progress.emit(5)
-            pcap = rdpcap(self.pcap)
-            date, value = [], []
-            for i in range(len(pcap)):
-                date.append(datetime.fromtimestamp(float(pcap[i].time)))
-                progress = int((i / len(pcap) * 100) * 0.45 + 5)
-                self.progress.emit(progress)
-
-            self.range = pd.date_range(date[0].replace(microsecond=0, second=0),
-                                       date[-1].replace(microsecond=0, second=0, minute=date[-1].minute + 1),
-                                       periods=100)
-            self.r_val = [0 for i in range(len(self.range))]
-
-            for d in range(len(date)):
-                for i in reversed(range(len(self.range))):
-                    if date[d] >= self.range[i]:
-                        self.r_val[i] += 1
-                        break
-                progress = int((d / len(date) * 100) + 50)
-                self.progress.emit(progress)
-        else:
-            self.progress.emit(50)
-            # self.range, self.r_val = [datetime(2000, 1, 1), datetime(2001, 1, 1)], [0, 0]
-            self.range, self.r_val = [], []
-            self.progress.emit(100)
-
-        self.data.emit([self.range, self.r_val])
-        self.finished.emit()
-
-
 class WorkspaceWindow(QMainWindow):
-    def __init__(self, workspace_object: Workspace, test_mode: bool = False,
-                 existing_flag: bool = False):
+    """Main GUI windows for the PacketVisualization software. A user can create/delete a project and add/delete a
+    dataset to a project. A user can then add/delete a PCAP to a dataset where analysis can be performed on the whole
+    dataset or selected packets.
+    """
+    QApplication.setAttribute(Qt.AA_EnableHighDpiScaling, True)
+    app = QApplication(sys.argv)
+    logo = os.path.join(os.path.dirname(__file__), "images", "logo.png")
+    app.setWindowIcon(QIcon(logo))
+
+    def __init__(self, workspace_path: str, existing_flag: bool = False) -> None:
+        """Initialization function for a new workspace window.
+
+        :param workspace_path: directory where workspace is saved
+        :type workspace_path: str
+        :param existing_flag: flag set if opening an existing workspace
+        :type existing_flag: bool
+        """
         super().__init__()
+        self.eo = EntityOperations()
+        self.db = None
         self.icons = os.path.join(os.path.dirname(__file__), "images", "svg")
         self.logo = os.path.join(os.path.dirname(__file__), "images", "logo.png")
-        self.workspace_object = workspace_object
-        self.test_mode = test_mode
-
+        self.test_mode = False
+        if existing_flag:
+            self.workspace_object = Load().open_zip(workspace_path)
+            restore_path = os.path.join(self.workspace_object.dump_path,self.workspace_object.name)
+            print(restore_path)
+            self.eo.restore_db(self.workspace_object.name, restore_path)
+            self.db = self.eo.set_db(self.workspace_object.name)
+        else:
+            self.workspace_object = Workspace(name=os.path.basename(workspace_path),
+                                              location=os.path.dirname(workspace_path))
+            self.workspace_object.create_dump_dir(self.workspace_object.dump_path)
+            self.db = self.eo.create_db(self.workspace_object.name)  # create DB with workspace name
         self.setWindowTitle("PacketVisualizer - " + self.workspace_object.name)
         self.resize(1000, 600)
+
         # Docked widget for Project Tree
         self.project_tree = QTreeWidget()
         self.project_tree.setHeaderLabels(["Item Name", "Size", "DoC"])
@@ -92,19 +80,34 @@ class WorkspaceWindow(QMainWindow):
         self.dock_project_tree = QDockWidget("Project Tree Window", self)
         self.dock_project_tree.setWidget(self.project_tree)
         self.dock_project_tree.setFloating(False)
+
+        self.traced_dataset = None
+        self.traced_data = None
         # Docked widget for Bandwidth vs. Time Graph
-        self.plot_range = []
-        self.plot_values = []
-        self.pcap = ''
+        self.plot_x = []
+        self.plot_y = []
         self.fig_view = QWebEngineView()
-        self.create_plot()
+        self.fig_view.setHtml(create_plot(self.plot_x, self.plot_y))
         self.dock_plot = QDockWidget("Bandwidth vs. Time Window", self)
         self.dock_plot.setWidget(self.fig_view)
         self.dock_plot.setFloating(False)
 
+        # Docked widget for Classifier
+        self.classifier_plot_view = QWebEngineView()
+        # TODO: X and Y data is going to be provided by Classifier class, once that happens we can fix this.
+        data_frame = pd.DataFrame()
+        data_frame['instance_number'] = [3, 2, 1]
+        data_frame['cluster'] = [1, 2, 3]
+        self.create_classifier_plot(data_frame)
+        self.classifier_window = QDockWidget("Cluster per instance", self)
+        self.classifier_window.setWidget(self.classifier_plot_view)
+        self.classifier_window.setFloating(False)
+        self.classifier_window.close()
+
         self.setCentralWidget(self.dock_project_tree)
         self.addDockWidget(Qt.LeftDockWidgetArea, self.dock_project_tree)
         self.addDockWidget(Qt.RightDockWidgetArea, self.dock_plot)
+        self.addDockWidget(Qt.RightDockWidgetArea, self.classifier_window)
 
         self._create_actions()
         self._create_menu_bar()
@@ -112,17 +115,21 @@ class WorkspaceWindow(QMainWindow):
         self._connect_actions()
         self._create_status_bar()
 
+        self.context = DbContext()
+        # self.db = self.context.db  # here
+        self.controller = Controller()
+
         if existing_flag:
-            self.workspace_object = Load().open_zip(
-                os.path.join(workspace_object.location, workspace_object.name + ".zip"))
             self.generate_existing_workspace()
 
-    def _create_actions(self):
-        # File Menu Actions
-        self.newWorkspaceAction = QAction("&New Workspace", self)
-        self.newWorkspaceAction.setStatusTip("Create a new workspace")
-        self.newWorkspaceAction.setToolTip("Create a new workspace")
+    def run(self):
+        self.show()
+        sys.exit(self.app.exec_())
 
+    def _create_actions(self) -> None:
+        """Creates all actions that will be used in the application
+        """
+        # File Menu Actions
         self.newProjectAction = QAction(QIcon(os.path.join(self.icons, "add-circle.svg")), "New &Project", self)
         self.newProjectAction.setShortcut("Ctrl+N")
         self.newProjectAction.setStatusTip("Create a new project")
@@ -138,9 +145,13 @@ class WorkspaceWindow(QMainWindow):
 
         self.filterWiresharkAction = QAction("Filter Wireshark", self)
 
-        self.openAction = QAction("&Existing Workspace", self)
-        self.openAction.setStatusTip("Open existing workspace")
-        self.openAction.setToolTip("Open existing workspace")
+        self.openNewAction = QAction("&New Workspace", self)
+        self.openNewAction.setStatusTip("Open a new workspace")
+        self.openNewAction.setToolTip("Open a new workspace")
+
+        self.openExistingAction = QAction("&Existing Workspace", self)
+        self.openExistingAction.setStatusTip("Open existing workspace")
+        self.openExistingAction.setToolTip("Open existing workspace")
 
         self.saveAction = QAction(QIcon(os.path.join(self.icons, "save.svg")), "&Save", self)
         self.saveAction.setShortcut("Ctrl+S")
@@ -188,6 +199,11 @@ class WorkspaceWindow(QMainWindow):
         self.gen_table_action.setStatusTip("View Packets in a PCAP")
         self.gen_table_action.setToolTip("View Packets in a PCAP")
 
+        # self.gen_table_action = QAction(QIcon(os.path.join(self.icons, "list.svg")), "&Packet Table", self)
+        self.classifier_action = QAction("&Classify Packets", self)
+        self.classifier_action.setStatusTip("Classify selected pcap data")
+        self.classifier_action.setToolTip("Classify selected pcap data")
+
         # Wireshark Menu Actions
         self.openWiresharkAction = QAction(QIcon(os.path.join(self.icons, "wireshark-icon.png")), "Open &Wireshark",
                                            self)
@@ -211,37 +227,48 @@ class WorkspaceWindow(QMainWindow):
         self.aboutAction = QAction("&About", self)
         self.aboutAction.setEnabled(False)
 
-    def _connect_actions(self):
+    def _connect_actions(self) -> None:
+        """Connects all actions to a method to be executed
+        """
         # Connect File actions
-        self.newWorkspaceAction.triggered.connect(self.new_workspace)
+        self.openNewAction.triggered.connect(self.open_new_workspace)
         self.newProjectAction.triggered.connect(self.new_project)
         self.newDatasetAction.triggered.connect(self.new_dataset)
         self.newPCAPAction.triggered.connect(self.new_pcap)
-        self.openAction.triggered.connect(self.open_workspace)
+        self.openExistingAction.triggered.connect(self.open_existing_workspace)
         self.saveAction.triggered.connect(self.save)
         self.exitAction.triggered.connect(self.exit)
         self.traceAction.triggered.connect(self.trace_dataset)
         self.exportCsvAction.triggered.connect(self.export_csv)
         self.exportJsonAction.triggered.connect(self.export_json)
+
         # Connect Edit actions
         self.cutAction.triggered.connect(self.cut_content)
         self.copyAction.triggered.connect(self.copy_content)
         self.pasteAction.triggered.connect(self.paste_content)
         self.deleteAction.triggered.connect(self.delete)
+
         # Connect View actions
         self.gen_table_action.triggered.connect(self.gen_table)
+        self.classifier_action.triggered.connect(self.display_classifier_options)
+
         # Connect Wireshark actions
         self.openWiresharkAction.triggered.connect(self.open_wireshark)
         self.filterWiresharkAction.triggered.connect(self.filter_wireshark)
+
         # Connect Windows actions
         self.openProjectTreeAction.triggered.connect(self.open_window_project_tree)
         self.openPlotAction.triggered.connect(self.open_window_plot)
+
         # Connect Help actions
         self.helpContentAction.triggered.connect(self.help_content)
         self.aboutAction.triggered.connect(self.about)
 
-    def _create_menu_bar(self):
+    def _create_menu_bar(self) -> None:
+        """Creates the top menu bar where all actions can be accessed
+        """
         menu_bar = self.menuBar()
+
         # File Menu
         file_menu = menu_bar.addMenu("&File")
         new_menu = file_menu.addMenu("&New...")
@@ -249,8 +276,8 @@ class WorkspaceWindow(QMainWindow):
         new_menu.addAction(self.newDatasetAction)
         new_menu.addAction(self.newPCAPAction)
         open_menu = file_menu.addMenu(QIcon(os.path.join(self.icons, "folder.svg")), "&Open...")
-        open_menu.addAction(self.newWorkspaceAction)
-        open_menu.addAction(self.openAction)
+        open_menu.addAction(self.openNewAction)
+        open_menu.addAction(self.openExistingAction)
         file_menu.addAction(self.saveAction)
         file_menu.addSeparator()
         file_menu.addAction(self.traceAction)
@@ -260,6 +287,7 @@ class WorkspaceWindow(QMainWindow):
         export_menu.addAction(self.exportJsonAction)
         file_menu.addSeparator()
         file_menu.addAction(self.exitAction)
+
         # Edit Menu
         edit_menu = menu_bar.addMenu("&Edit")
         edit_menu.addAction(self.cutAction)
@@ -267,43 +295,49 @@ class WorkspaceWindow(QMainWindow):
         edit_menu.addAction(self.pasteAction)
         edit_menu.addSeparator()
         edit_menu.addAction(self.deleteAction)
+
         # View Menu
         view_menu = menu_bar.addMenu("&View")
         view_menu.addAction(self.gen_table_action)
+        view_menu.addAction(self.classifier_action)
+
         # Wireshark Menu
         wireshark_menu = menu_bar.addMenu('Wire&shark')
         wireshark_menu.addAction(self.openWiresharkAction)
+
         # Window Menu
         windows_menu = menu_bar.addMenu('&Window')
         windows_menu.addAction(self.openPlotAction)
         windows_menu.addAction(self.openProjectTreeAction)
+
         # Help Menu
         help_menu = menu_bar.addMenu("&Help")
         help_menu.addAction(self.helpContentAction)
         help_menu.addAction(self.aboutAction)
 
-    def _create_tool_bar(self):
+    def _create_tool_bar(self) -> None:
+        """Creates a tool bar for quick access to come actions
+        """
         file_tool_bar = QToolBar("File")
         self.addToolBar(Qt.LeftToolBarArea, file_tool_bar)
         file_tool_bar.addAction(self.newProjectAction)
         file_tool_bar.addAction(self.saveAction)
         file_tool_bar.addAction(self.deleteAction)
 
-    def _create_status_bar(self):
-        self.statusbar = self.statusBar()
-        self.statusbar.showMessage("Ready", 3000)
+    def _create_status_bar(self) -> None:
+        """Creates a status bar to keep track of task progress
+        """
+        self.status_bar = self.statusBar()
+        self.status_bar.showMessage("Ready", 3000)
 
         self.progressbar = QProgressBar()
         self.progressbar.setFixedSize(250, 12)
         self.progressbar.setTextVisible(False)
-        self.progressbar.setStyleSheet("QProgressBar::chunk "
-                                       "{"
-                                       "background-color: grey;"
-                                       "}")
-        self.statusbar.addPermanentWidget(self.progressbar)
+        self.progressbar.setStyleSheet("QProgressBar::chunk {background-color: grey;}")
+        self.status_bar.addPermanentWidget(self.progressbar)
         self.progressbar.hide()
 
-    def contextMenuEvent(self, event):
+    def contextMenuEvent(self, event) -> None:
         # Right Click Menu
         menu = QMenu(self.project_tree)
 
@@ -317,6 +351,9 @@ class WorkspaceWindow(QMainWindow):
                 menu.addAction(self.traceAction)
                 menu.addAction(self.openWiresharkAction)
                 menu.addAction(self.filterWiresharkAction)
+                menu.addAction(self.classifier_action)
+                view_menu = menu.addMenu("View")
+                view_menu.addAction(self.gen_table_action)
                 export_menu = menu.addMenu("Export")
                 export_menu.addAction(self.exportCsvAction)
                 export_menu.addAction(self.exportJsonAction)
@@ -344,21 +381,6 @@ class WorkspaceWindow(QMainWindow):
 
         menu.exec(event.globalPos())
 
-    def new_workspace(self, file=None):
-        # Logic for creating a new workspace
-        try:
-            if not self.test_mode:
-                file = QFileDialog.getSaveFileName(caption="Choose Workspace location")[0]
-
-            if file != '':
-                new_workspace_object = Workspace(name=os.path.basename(file), location=os.path.dirname(file))
-                self.workspace = WorkspaceWindow(new_workspace_object)
-                self.workspace.show()
-                return True
-        except Exception:
-            traceback.print_exc()
-            return False
-
     def new_project(self, text=None):
         # Logic for creating a new project
         if not self.test_mode:
@@ -408,6 +430,9 @@ class WorkspaceWindow(QMainWindow):
                         pcap_item.setText(0, pcap_name)
                         pcap_item.setData(0, Qt.UserRole, new_pcap)
                         child_item.addChild(pcap_item)
+
+                        mytable = self.db[dataset.name]
+                        self.eo.insert_packets(new_pcap.json_file, mytable, dataset.name, new_pcap.name)
                     else:
                         child_item.parent().removeChild(child_item)
                         p.del_dataset(dataset)
@@ -442,36 +467,42 @@ class WorkspaceWindow(QMainWindow):
                     pcap_item.setText(0, pcap_name)
                     pcap_item.setData(0, Qt.UserRole, new_pcap)
                     dataset_item.addChild(pcap_item)
-                if self.pcap != "":
-                    self.update_plot()
+
+                    mytable = self.db[d.name]
+                    self.eo.insert_packets(new_pcap.json_file, mytable, d.name, new_pcap.name)
+                if self.traced_dataset:
+                    self.update_traced_data()
         except Exception:
             print("Error loading this pcap")
             traceback.print_exc()
 
-    def open_workspace(self):
-        # Logic for opening an existing project
-        try:
-            if not self.test_mode:
-                file_filter = "zip(*.zip)"
-                path = QFileDialog.getOpenFileName(caption="Open existing Workspace", filter=file_filter)[0]
+    def open_new_workspace(self) -> None:
+        """Prompts user to select save location and opens up a new workspace window
+        """
+        directory = QFileDialog.getSaveFileName(caption="Choose Workspace location")[0]
 
-                if path != "":
-                    if not self.test_mode:
-                        workspace_object = Workspace(name=os.path.basename(path.replace(".zip", "")),
-                                                     location=os.path.dirname(path))
-                        self.workspace = WorkspaceWindow(workspace_object, existing_flag=True)
-                        self.workspace.show()
-                        return True
-        except Exception:
-            traceback.print_exc()
+        if directory != '':
+            self.new_window = WorkspaceWindow(workspace_path=directory)
+            self.new_window.show()
+
+    def open_existing_workspace(self) -> None:
+        """Prompts user to select saved zip file and opens an existing workspace window
+        """
+        file_filter = "zip(*.zip)"
+        file_path = QFileDialog.getOpenFileName(caption="Open existing Workspace", filter=file_filter)[0]
+
+        if file_path != "":
+            if not self.test_mode:
+                self.new_window = WorkspaceWindow(workspace_path=file_path, existing_flag=True)
+                self.new_window.show()
 
     def trace_dataset(self):
         if self.project_tree.selectedItems() and type(
                 self.project_tree.selectedItems()[0].data(0, Qt.UserRole)) is Dataset:
             dataset_item = self.project_tree.selectedItems()[0]
             d = dataset_item.data(0, Qt.UserRole)
-            self.pcap = d.mergeFilePath
-            self.update_plot()
+            self.traced_dataset = d
+            self.update_traced_data()
 
     def export_csv(self):
         # Logic to export dataset or pcap to CSV
@@ -508,9 +539,9 @@ class WorkspaceWindow(QMainWindow):
                                                               '-E header=y -E '
                                                               'separator=, -E quote=d -E '
                                                               'occurrence=f > ' + output_file)
-                    self.statusbar.showMessage("Export CSV Successful", 3000)
+                    self.status_bar.showMessage("Export CSV Successful", 3000)
                 else:
-                    self.statusbar.showMessage("No Dataset/PCAP selected to Export", 3000)
+                    self.status_bar.showMessage("No Dataset/PCAP selected to Export", 3000)
                 return True
         except Exception:
             traceback.print_exc()
@@ -530,24 +561,29 @@ class WorkspaceWindow(QMainWindow):
                     input_file = pcap.path
 
                 if input_file != '':
-                    output_file = QFileDialog.getSaveFileName(caption="Choose Output location", filter=".json (*.json)")[0]
+                    output_file = \
+                        QFileDialog.getSaveFileName(caption="Choose Output location", filter=".json (*.json)")[0]
 
                     if pf.system() == "Windows":
-                        os.system('cd "C:\Program Files\Wireshark" & tshark -r ' + input_file + ' > ' + output_file)
+                        os.system(
+                            'cd "C:\Program Files\Wireshark" & tshark -r ' + input_file + ' -T json > ' + output_file)
                     if pf.system() == "Linux":
-                        os.system('tshark -r ' + input_file + ' > ' + output_file)
-                    self.statusbar.showMessage("Export JSON Successful", 3000)
+                        os.system('tshark -r ' + input_file + ' -T json > ' + output_file)
+                    self.status_bar.showMessage("Export JSON Successful", 3000)
                 else:
-                    self.statusbar.showMessage("No Dataset/PCAP selected to Export", 3000)
+                    self.status_bar.showMessage("No Dataset/PCAP selected to Export", 3000)
                 return True
         except Exception:
             traceback.print_exc()
 
-    def save(self):
-        # Logic for creating a new project
-        self.statusbar.showMessage("Saving...")
+    def save(self) -> None:
+        """ Function to save a workspace and all associated data
+        """
+        self.status_bar.showMessage("Saving...")
+
+        self.eo.dump_db(self.workspace_object.name, self.workspace_object.dump_path)
         self.workspace_object.save()
-        self.statusbar.showMessage("Saved", 3000)
+        self.status_bar.showMessage("Saved", 3000)
 
     def delete(self, project_item=None, dataset_item=None, pcap_item=None):
         # Logic for deleting an item
@@ -567,9 +603,10 @@ class WorkspaceWindow(QMainWindow):
                 for p in self.workspace_object.project:
                     for d in p.dataset:
                         if d.name == dataset_item.text(0):
-                            if d.mergeFilePath == self.pcap:
-                                self.pcap = ""
-                                self.update_plot()
+                            if self.traced_dataset and d.name == self.traced_dataset.name:
+                                self.traced_dataset = None
+                                self.eo.delete_collection(self.db[d.name])
+                                self.update_traced_data()
                             p.del_dataset(old=d)
                             dataset_item.parent().removeChild(dataset_item)
             # Deleting a pcap
@@ -582,13 +619,15 @@ class WorkspaceWindow(QMainWindow):
                             if cap.name == pcap_item.text(0):
                                 d.del_pcap(cap)
                                 pcap_item.parent().removeChild(pcap_item)
-                if self.pcap != "":
-                    self.update_plot()
+                                self.eo.delete_packets(self.db[d.name], "parent_pcap", cap.name)
+                if self.traced_dataset:
+                    self.update_traced_data()
         else:
             return False
 
-    def exit(self):
-        # Logic for exiting the program
+    def exit(self) -> None:
+        """Exit function that calls the close() function to stop the application
+        """
         self.close()
 
     def cut_content(self):
@@ -650,11 +689,21 @@ class WorkspaceWindow(QMainWindow):
                                      QMessageBox.Yes | QMessageBox.No | QMessageBox.Cancel, QMessageBox.Yes)
 
         if reply == QMessageBox.Yes:
-            self.workspace_object.save()
+            self.save()
+            self.eo.remove_db(self.workspace_object.name)
             self.workspace_object.__del__()
+            if os.path.exists("tEmPpCaP.pcap"):
+                os.remove("tEmPpCaP.pcap")
+            if os.path.exists("tEmPmErGeCaP.pcap"):
+                os.remove("tEmPmErGeCaP.pcap")
             event.accept()
         elif reply == QMessageBox.No:
+            self.eo.remove_db(self.workspace_object.name)
             self.workspace_object.__del__()
+            if os.path.exists("tEmPpCaP.pcap"):
+                os.remove("tEmPpCaP.pcap")
+            if os.path.exists("tEmPmErGeCaP.pcap"):
+                os.remove("tEmPmErGeCaP.pcap")
             event.accept()
         else:
             event.ignore()
@@ -733,7 +782,18 @@ class WorkspaceWindow(QMainWindow):
                 pcap_item = self.project_tree.selectedItems()[0]
                 pcap_obj = pcap_item.data(0, Qt.UserRole)
 
-                table = table_gui(pcap_obj, self.progressbar)
+                table = table_gui(pcap_obj, self.progressbar, self.db, self)
+                self.dock_table = QDockWidget("Packet Table", self)
+                self.dock_table.setWidget(table)
+                self.dock_table.setFloating(False)
+                self.addDockWidget(Qt.BottomDockWidgetArea, self.dock_table)
+
+            if self.project_tree.selectedItems() and type(
+                    self.project_tree.selectedItems()[0].data(0, Qt.UserRole)) is Dataset:
+                dataset_item = self.project_tree.selectedItems()[0]
+                dataset_obj = dataset_item.data(0, Qt.UserRole)
+
+                table = table_gui(dataset_obj, self.progressbar, self.db, self)
                 self.dock_table = QDockWidget("Packet Table", self)
                 self.dock_table.setWidget(table)
                 self.dock_table.setFloating(False)
@@ -759,6 +819,7 @@ class WorkspaceWindow(QMainWindow):
             return None, None, full_path
 
     def generate_existing_workspace(self):
+        # here -----------------------------------Load Dump---------------------------------------------------
         for p in self.workspace_object.project:
             project_item = QTreeWidgetItem(self.project_tree)
             project_item.setText(0, p.name)
@@ -773,87 +834,88 @@ class WorkspaceWindow(QMainWindow):
                     pcap_item.setText(0, cap.name)
                     pcap_item.setData(0, Qt.UserRole, cap)
                     dataset_item.addChild(pcap_item)
+
+
+
+
         return True
 
-    def show_qt(self, fig):
+    def show_classifier_qt(self, fig):
         raw_html = '<html><head><meta charset="utf-8" />'
         raw_html += '<script src="https://cdn.plot.ly/plotly-latest.min.js"></script></head>'
         raw_html += '<body>'
         raw_html += po.plot(fig, include_plotlyjs=False, output_type='div')
         raw_html += '</body></html>'
 
-        if self.fig_view is None:
-            self.fig_view = QWebEngineView()
+        if self.classifier_plot_view is None:
+            self.classifier_plot_view = QWebEngineView()
         # setHtml has a 2MB size limit, need to switch to setUrl on tmp file
         # for large figures.
-        self.fig_view.setHtml(raw_html)
+        self.classifier_plot_view.setHtml(raw_html)
 
-    def create_plot(self):
-        # Create figure
-        fig = go.Figure()
-        fig.add_trace(go.Scatter(x=self.plot_range, y=self.plot_values))
+    def create_classifier_plot(self, df):
+        fig = px.scatter(df, x='cluster', y='instance_number',
+                         color='cluster', color_continuous_scale=px.colors.sequential.Bluered_r)
+        # fig.show()
+        self.show_classifier_qt(fig)
 
-        # Set title
-        fig.update_layout(title_text="Bandwidth vs. Time")
+    def update_traced_data(self):
+        """Updates the traced data when a packet is added or deleted
+        """
+        if self.traced_dataset:
+            collection = self.db[self.traced_dataset.name]
+            query = {'parent_dataset': self.traced_dataset.name}
+            self.traced_data = list(collection.find(query))
+        else:
+            self.traced_data = None
+        self.update_plot()
 
-        # Add range slider
-        fig.update_layout(
-            xaxis=dict(
-                rangeselector=dict(
-                    buttons=list([
-                        dict(step="all")
-                    ])
-                ),
-                rangeslider=dict(
-                    visible=True
-                ),
-                type="date"
-            )
-        )
-        self.show_qt(fig)
+    def report_progress(self, n: int) -> None:
+        """Updates the progress bar from worker signal
 
-    def reportProgress(self, n):
+        :param n: an integer between 0-100 representing the percentage completed
+        :type n: int
+        """
         self.progressbar.setValue(n)
 
-    def report_plot_data(self, n):
-        self.plot_range = n[0]
-        self.plot_values = n[1]
+    def report_plot_data(self, n: list) -> None:
+        """Updates bandwidth plot x & y values from worker signal
+
+        :param n: a list of lists, (list of x values and list of y values)
+        :type n: list
+        """
+        self.plot_x = n[0]
+        self.plot_y = n[1]
 
     def update_plot(self):
+        """ Creates a new thread to update bandwidth vs. time graph
+        """
+        # Step 1: Begin showing progress bar
         self.progressbar.show()
-
         # Step 2: Create a QThread object
-        self.thread = QThread()
+        self.thread_1 = QThread()
         # Step 3: Create a worker object
-        self.worker = Worker(self.plot_range, self.plot_values, self.pcap)
+        self.worker_1 = PlotWorker(self.traced_data)
         # Step 4: Move worker to the thread
-        self.worker.moveToThread(self.thread)
+        self.worker_1.moveToThread(self.thread_1)
         # Step 5: Connect signals and slots
-        self.thread.started.connect(self.worker.run)
-        self.worker.finished.connect(self.thread.quit)
-        self.worker.finished.connect(self.worker.deleteLater)
-        self.thread.finished.connect(self.thread.deleteLater)
-        self.worker.progress.connect(self.reportProgress)
-        self.worker.data.connect(self.report_plot_data)
+        self.thread_1.started.connect(self.worker_1.run)
+        self.worker_1.finished.connect(self.thread_1.quit)
+        self.worker_1.finished.connect(self.worker_1.deleteLater)
+        self.thread_1.finished.connect(self.thread_1.deleteLater)
+        self.worker_1.progress.connect(self.report_progress)
+        self.worker_1.data.connect(self.report_plot_data)
         # Step 6: Start the thread
-        self.thread.start()
+        self.thread_1.start()
 
         # Final resets
-        self.thread.finished.connect(
-            lambda: self.progressbar.setValue(0)
-        )
-
-        self.thread.finished.connect(
-            lambda: self.progressbar.hide()
-        )
-
-        self.thread.finished.connect(
-            lambda: self.create_plot()
-        )
+        self.thread_1.finished.connect(lambda: self.progressbar.setValue(0))
+        self.thread_1.finished.connect(lambda: self.progressbar.hide())
+        self.thread_1.finished.connect(lambda: self.fig_view.setHtml(create_plot(self.plot_x, self.plot_y)))
 
     def filter_wireshark(self):
 
-        if self.project_tree.selectedItems(): # and self.check_if_item_is(self.project_tree.selectedItems()[0], "Dataset"):
+        if self.project_tree.selectedItems():  # and self.check_if_item_is(self.project_tree.selectedItems()[0], "Dataset"):
 
             if self.test_mode == False:
                 dataset_item = self.project_tree.selectedItems()[0]
@@ -861,3 +923,34 @@ class WorkspaceWindow(QMainWindow):
                 for d in p.dataset:
                     if d.name == dataset_item.text(0):
                         ui = filter_gui.filter_window(d.mergeFilePath, self.project_tree, self.workspace_object)
+
+    def display_classifier_options(self):
+        results = pd.DataFrame()
+        try:
+            if self.project_tree.selectedItems():
+                if type(self.project_tree.selectedItems()[0].data(0, Qt.UserRole)) is Dataset:
+                    dataset_item = self.project_tree.selectedItems()[0]
+                    dataset = dataset_item.data(0, Qt.UserRole)
+                    results = self.controller.classify_dataset(dataset.name)
+        except:
+            raise ''
+
+        self.create_classifier_plot(results)
+        self.classifier_window.show()
+        return
+
+
+if __name__ == "__main__":
+    args = len(sys.argv)
+    try:
+        path = os.path.realpath(sys.argv[1])
+        if args == 2:
+            ui = WorkspaceWindow(path)
+        elif args == 3:
+            if sys.argv[2] == 'True':
+                ui = WorkspaceWindow(path, True)
+            else:
+                ui = WorkspaceWindow(path)
+        ui.run()
+    except Exception:
+        traceback.print_exc()
